@@ -60,9 +60,12 @@ class _Future(ndb.model.Model):
 
     def intask(self, nameprefix, f, *args, **kwargs):
         taskkwargs = self.get_taskkwargs()
-        
-        name = "%s-%s" % (nameprefix, self.key.id())
-        taskkwargs["name"] = name
+
+        if nameprefix:
+            name = "%s-%s" % (nameprefix, self.key.id())
+            taskkwargs["name"] = name
+        elif taskkwargs.get("name"):
+            del taskkwargs["name"]
         taskkwargs["transactional"] = False
         
         @task(**taskkwargs)
@@ -72,7 +75,7 @@ class _Future(ndb.model.Model):
         try:
             # run the wrapper task, and if it fails due to a name clash just skip it (it was already kicked off by an earlier
             # attempt to construct this future).
-            logging.debug("about to run task %s" % name)
+#             logging.debug("about to run task %s" % name)
             dof()
         except taskqueue.TombstonedTaskError:
             logging.debug("skip adding task %s (already been run)" % name)
@@ -175,7 +178,7 @@ class _Future(ndb.model.Model):
         if onprogressf:
             def doonprogressf():
                 onprogressf(self.key)
-            self.intask("onprogress", doonprogressf)
+            self.intask(None, doonprogressf)
             
     def get_runtime(self):
         if self.runtimesec:
@@ -439,7 +442,7 @@ def DefaultOnFailure(futurekey):
         except Exception, ex:
             parentfutureobj.set_failure(ex)
 
-def GenerateOnAllChildSuccess(parentkey, initialvalue, combineresultf):
+def GenerateOnAllChildSuccess(parentkey, initialvalue, combineresultf, failonerror=True):
     def OnAllChildSuccess():
         logging.debug("Enter GenerateOnAllChildSuccess: %s" % parentkey)
         parentfuture = parentkey.get() if parentkey else None
@@ -475,7 +478,10 @@ def GenerateOnAllChildSuccess(parentkey, initialvalue, combineresultf):
                          
                 if error:
                     logging.warning("Internal error, child has error in OnAllChildSuccess: %s" % error)
-                    parentfuture.set_failure(error)
+                    if failonerror:
+                        parentfuture.set_failure(error)
+                    else:
+                        raise error
                 elif finished:
                     logging.debug("result: %s" % result)
                     parentfuture.set_success(result)#(result, initialamount, keyrange))
@@ -540,7 +546,7 @@ def OnProgressF(futurekey):
 def get_children(futurekey):
     if futurekey:
         ancestorkey = ndb.Key(futurekey.kind(), futurekey.id())
-        return [childfuture for childfuture in _Future.query(ancestor=ancestorkey) if ancestorkey == childfuture.key.parent()]
+        return [childfuture for childfuture in _Future.query(ancestor=ancestorkey).order(_Future.stored) if ancestorkey == childfuture.key.parent()]
     else:
         return []
 
@@ -714,7 +720,7 @@ def future(f=None, parentkey=None,
             else:
                 break # do we need this? Don't think so
 
-    logging.debug("fffff")
+    logging.debug("about to call runfuture")
     return runfuture
 
 def GetFutureAndCheckReady(futurekey):
@@ -723,3 +729,106 @@ def GetFutureAndCheckReady(futurekey):
         raise Exception("Future not ready for result, retry")
     return futureobj
 
+# fsf = futuresequencefunction
+# different from future function because it has a "results" argument, a list.
+def futuresequence(fsfseq, parentkey = None, onsuccessf=None, onfailuref=None, onallchildsuccessf=None, onprogressf=None, weight=None, timeoutsec=1800, maxretries=None, futurenameprefix=None, **taskkwargs):
+    flist = list(fsfseq)
+     
+    taskkwargs["futurename"] = "%s (top level)" % futurenameprefix if futurenameprefix else "sequence"
+     
+    @future(parentkey = parentkey, onsuccessf = onsuccessf, onfailuref = onfailuref, onallchildsuccessf=onallchildsuccessf, onprogressf = onprogressf, weight=weight, timeoutsec=timeoutsec, maxretries=maxretries, **taskkwargs)
+    def toplevel(futurekey, *args, **kwargs):
+        def childonsuccessforindex(index, results):
+            def childonsuccess(childfuturekey):
+                logging.debug("Enter childonsuccess")
+                childfuture = GetFutureAndCheckReady(childfuturekey)
+                 
+                try:    
+                    result = childfuture.get_result()
+                except Exception, ex:
+                    toplevelfuture = futurekey.get()
+                    if toplevelfuture:
+                        toplevelfuture.set_failure(ex)
+                    else:
+                        raise Exception("Can't load toplevel future for failure")
+                else:
+                    newresults = results + [result]
+                    islast = (index == (len(flist) - 1))
+                     
+                    if islast:
+                        toplevelfuture = futurekey.get()
+                        if toplevelfuture:
+                            toplevelfuture.set_success_and_readyforesult(newresults)
+                        else:
+                            raise Exception("Can't load toplevel future for success")
+                    else:
+                        taskkwargs["futurename"] = "%s [%s]" % (futurenameprefix if futurenameprefix else "-", index+1)
+                        future(flist[index+1], parentkey=futurekey, onsuccessf=childonsuccessforindex(index+1, newresults), weight=weight/len(flist) if weight else None, timeoutsec=timeoutsec, maxretries=maxretries, **taskkwargs)(newresults)
+            return childonsuccess
+ 
+        taskkwargs["futurename"] = "%s [0]" % (futurenameprefix if futurenameprefix else "sequence")
+        future(flist[0], parentkey=futurekey, onsuccessf=childonsuccessforindex(0, []), weight=weight/len(flist) if weight else None, timeoutsec=timeoutsec, maxretries=maxretries, **taskkwargs)([]) # empty list of results 
+                 
+        raise FutureNotReadyForResult("sequence started")
+ 
+    return toplevel
+
+def futureparallel(ffseq, parentkey = None, onsuccessf=None, onfailuref=None, onallchildsuccessf=None, onprogressf=None, weight=None, timeoutsec=1800, maxretries=None, futurenameprefix=None, **taskkwargs):
+    flist = list(ffseq)
+     
+    taskkwargs["futurename"] = "%s (top level)" % futurenameprefix if futurenameprefix else "parallel"
+     
+    @future(parentkey = parentkey, onsuccessf = onsuccessf, onfailuref = onfailuref, onallchildsuccessf=onallchildsuccessf, onprogressf = onprogressf, weight=weight, timeoutsec=timeoutsec, maxretries=maxretries, **taskkwargs)
+    def toplevel(futurekey, *args, **kwargs):
+        def OnAllChildSuccess():
+            logging.debug("Enter OnAllChildSuccess: %s" % futurekey)
+            parentfuture = futurekey.get() if futurekey else None
+            if parentfuture and not parentfuture.has_result():
+                if not parentfuture.initialised or not parentfuture.readyforresult:
+                    raise Exception("Parent not initialised, retry")
+                
+                @ndb.transactional()
+                def get_children_trans():
+                    return get_children(parentfuture.key)
+                children = get_children_trans()
+                
+                logging.debug("children: %s" % [child.key for child in children])
+                if children:
+                    result = []
+                    error = None
+                    finished = True
+                    for childfuture in children:
+                        logging.debug("childfuture: %s" % childfuture.key)
+                        if childfuture.has_result():
+                            try:
+                                childresult = childfuture.get_result()
+                                logging.debug("childresult(%s): %s" % (childfuture.status, childresult))
+                                result += [childfuture.get_result()]
+                                logging.debug("intermediate result:%s" % result)
+                            except Exception, ex:
+                                logging.debug("haserror:%s" % repr(ex))
+                                error = ex
+                                break
+                        else:
+                            logging.debug("noresult")
+                            finished = False
+                             
+                    if error:
+                        logging.warning("Internal error, child has error in OnAllChildSuccess: %s" % error)
+                        parentfuture.set_failure(error)
+                    elif finished:
+                        logging.debug("result: %s" % result)
+                        parentfuture.set_success(result)
+                    else:
+                        logging.debug("child not finished in OnAllChildSuccess, skipping")
+                else:
+                    logging.warning("Internal error, parent has no children in OnAllChildSuccess")
+                    parentfuture.set_failure(Exception("no children found"))
+        
+        for ix, ff in enumerate(flist):
+            taskkwargs["futurename"] = "%s [%s]" % (futurenameprefix if futurenameprefix else "parallel", ix)
+            future(ff, parentkey=futurekey, onallchildsuccessf = OnAllChildSuccess, weight=weight/len(flist) if weight else None, timeoutsec=timeoutsec, maxretries=maxretries, **taskkwargs)()
+                 
+        raise FutureReadyForResult("parallel started")
+ 
+    return toplevel
